/***************************************************************************
 *   Copyright (C) 2004 by erss                                            *
 *   erss@mail.ru                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "reer.h"
#include "abbr.h"

appler::appler(const QString& configFile, Type type, int actions, QWidget *parent, const char *name) 
    : KPanelApplet(configFile, type, actions, parent, name)
{
  codec = QTextCodec::codecForName("KOI8-R");
  
  setBackgroundMode(X11ParentRelative);
  
  input = new KHistoryCombo(this);
  input->setFocus();
  input->clearEdit();
  input->lineEdit()->installEventFilter( this );
  input->setDuplicatesEnabled( FALSE );
  input->setMaxCount(100);
  QToolTip::add( input, i18n("You can use regular expression for searching") );
  
  connect(input, SIGNAL(activated(const QString&)), SLOT(showWord(const QString&)));
  
  QSettings s;
  QStringList list = s.readListEntry( "/reer/main/wordsList" );
  input->setHistoryItems( list );
  
  reloadConfig();
}

appler::~appler()
{
  QSettings s;
  s.writeEntry( "/reer/main/wordsList", input->historyItems() );
}

void appler::reloadConfig()
{
  QSettings s;
  QString sDir = s.readEntry( "/reer/main/dictdir", "/usr/local/share/dict" );
  QStringList lst = s.readListEntry( "/reer/main/active_dicts" );
  
  QDir dir( sDir, "*.dict" );
  if ( dir.exists() ) {
    maDictionary tmpDict;
    dicts.clear();
    for ( uint i=0; i<dir.count(); i++ ) {
      if ( ! lst.grep( QRegExp(dir[i]) ).empty() ) {
        tmpDict.setName( sDir+"/"+dir[i] );
        dicts[dir[i]] = tmpDict;
      }
    }
  }

  useRegExp = s.readBoolEntry( "/reer/main/useRegExp", FALSE );
}

void appler::about()
{
  KMessageBox::information(0, i18n("RE-ER Dictionary KDE Applet v.0.2"), i18n("About") );
}


void appler::help()
{
  KMessageBox::informationList(0, i18n("<b>Abbreviations</b>"),
         QStringList::split( '\n',codec->toUnicode( abbrs ) ), i18n("Help") );
}

void appler::preferences()
{
  setup dlg;
  if ( dlg.exec()==QDialog::Accepted ) reloadConfig();
}

int appler::widthForHeight(int height) const
{
  return 150;
}

int appler::heightForWidth(int width) const
{
  return height();
}

void appler::resizeEvent(QResizeEvent *e)
{
  input->reparent(this, QPoint(0,0), true);
  int inputVOffset = height() - input->sizeHint().height() - 2;
  input->setGeometry(0, inputVOffset, width(), input->sizeHint().height());
  updateGeometry();
}

bool appler::eventFilter( QObject *o, QEvent * e)
{
  if ( ( e->type() == QEvent::MouseButtonRelease ) ||  
       ( e->type() == QEvent::MouseButtonPress ) ) {
     emit requestFocus();
  }
  return KPanelApplet::eventFilter( o, e );
}

void appler::showWord(const QString& word)
{
  QString article("");
  input->addToHistory(word);
  input->clearEdit();
  QRegExp defRE;
  if ( useRegExp ) {
    defRE = QRegExp( word.lower() );
  } else {
    defRE = QRegExp( "^"+word.lower().stripWhiteSpace()+"\\w*" );
  }
  QStringList totallst;
  for ( mapDict::iterator i = dicts.begin(); i != dicts.end(); i++ )
    totallst += i.data().grep( defRE );

  if ( !totallst.empty() ) {
    wordList *w = new wordList( totallst, this, word/*.latin1()*/, Qt::WDestructiveClose | Qt::WType_TopLevel );
    connect( w, SIGNAL( selected( const QString&, const QString& ) ), 
             this, SLOT( slShowArticle( const QString&, const QString& ) ) );
    w->show();
//  wordWindow *w = new wordWindow( 0, word.latin1(), WStyle_NoBorder|WStyle_StaysOnTop|WX11BypassWM|Qt::WDestructiveClose );
  }
}

void appler::slShowArticle(const QString& word,const QString& dictionary )
{
  QStringList lst= dicts[dictionary].grepp( QRegExp( "^"+word+"@\\d*") );
  if ( !lst.empty() ) {
    QFile dict;
    dict.setName( dicts[dictionary].getName() );
    if ( dict.open( IO_ReadOnly ) ) {
      QString article("");
      dict.at( lst.first().right( lst.first().length() - lst.first().find(QRegExp("@"),0)-1 ).toInt() );
      QTextStream ts( &dict );
      ts.setEncoding(QTextStream::UnicodeUTF8);
      article = ts.readLine();
//      dict.readLine( article, dict.size() );
      wordWindow *w = new wordWindow( article , this, word.latin1(), Qt::WDestructiveClose | WType_TopLevel );
      w->setCaption( word );
      w->show(); 
      w->setActiveWindow();
    }
  }
}

extern "C"
{
  KPanelApplet* init( QWidget *parent, const QString configFile)
  {
    KGlobal::locale()->insertCatalogue("appler");
      return new appler(configFile, KPanelApplet::Stretch,
                        KPanelApplet::About | KPanelApplet::Help | KPanelApplet::Preferences,
                        parent, "appler");

  }
}

//////////////////////////  wordWindow  ////////////////////////

wordWindow::wordWindow( QString txt, QWidget *parent, const char *name, WFlags f )
  : QMainWindow( parent, name, f )
{
  QSettings s;
  
  text = new QTextBrowser(this);
  
  QRegExp rex("(\\d+>|\\w+>|\\d+\\.|_[IVX]+)");
  int pos=txt.find( rex, 0 );
  while ( pos!=-1 ) {
    txt.insert( pos, "<p>" );
    pos=pos+8;
    pos=txt.find( rex, pos );
  }
  enWord = txt.left( txt.find(QRegExp( "  " ),0) );
  txt.replace( enWord, "<font color=darkblue><b>"+enWord+"</b></font>" );
  QString sAudio = s.readEntry( "/reer/main/dir", "/usr/local/share/erss/sounds" )+"/"+enWord+".wav";
  if ( QFile::exists( sAudio ) ) 
    txt.insert( txt.find(QRegExp( "  " ),0)+1,
                QString("<a href=\"#play\"><img src=\"%1/sound.png\"></a>").arg( s.readEntry( "/reer/main/dictdir", "/usr/local/share/dict" ) ) );
  text->setText( txt );
  text->setReadOnly( TRUE );
  connect( text, SIGNAL( linkClicked( const QString & ) ),
            this, SLOT( slPlaySound( const QString & ) ) );
  setCentralWidget(text);

  resize( QSize( s.readNumEntry( "/reer/wordWindow/width", 200 ),
                 s.readNumEntry( "/reer/wordWindow/height", 100 ) ) );
                 
  if ( ( dispatcher = new KArtsDispatcher() ) ) {
     server = Arts::Reference("global:Arts_SoundServer");
  }
}

wordWindow::~wordWindow()
{
  QSettings s;
  s.writeEntry( "/reer/wordWindow/width", width() );
  s.writeEntry( "/reer/wordWindow/height", height() );
}

void wordWindow::keyPressEvent( QKeyEvent *e )
{
  switch( e->key() ) {
    case Qt::Key_Escape : {
      close();    
      break;
    }
    default: e->ignore();
  }
  QMainWindow::keyPressEvent( e );
}

void wordWindow::slPlaySound( const QString & )
{
  QSettings s;
  server.play( QString(s.readEntry( "/reer/main/dir", "" )+"/"+enWord+".wav").latin1() ); 
}

////////////////////// wordList ///////////////////////
wordList::wordList( QStringList lst, QWidget *parent, const char *name, WFlags f )
  : QMainWindow( parent, name, f )
{
  setCaption( name );
  lvWords = new QListView( this );
  lvWords->header()->hide();
  lvWords->addColumn("Word");
  lvWords->addColumn("Dictionary");
  lvWords->setSelectionMode( QListView::Single );
  lvWords->setAllColumnsShowFocus( TRUE );
  setCentralWidget(lvWords);
    
  for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it ) {
    QString s = *it;
    myListViewItem *item =new myListViewItem( lvWords, static_cast<QListViewItem*>( lvWords->lastItem() ) );
    item->setText( 0, s.left( s.find(QRegExp("#"),0) ) );
    item->setText( 1, s.right( s.length()-s.find( QRegExp("#") )-1) );
  };
  connect( lvWords, SIGNAL( doubleClicked( QListViewItem *, const QPoint &, int ) ),
            this, SLOT(slDoubleClicked( QListViewItem *, const QPoint &, int ) ) );
  QSettings s;
  resize( QSize( s.readNumEntry( "/reer/wordList/width", 100 ),
                 s.readNumEntry( "/reer/wordList/height", 200 ) ) );
}

wordList::~wordList()
{
  QSettings s;
  s.writeEntry( "/reer/wordList/width", width() );
  s.writeEntry( "/reer/wordList/height", height() );
}

void wordList::keyPressEvent( QKeyEvent *e )
{
  switch( e->key() ) {
    case Qt::Key_Escape : {
      close();    
      break;
    }
    case Qt::Key_Enter : case Qt::Key_Return : case Qt::Key_Space : {
      QListViewItem *item = lvWords->currentItem();
      if ( item ) emit selected( item->text(0),item->text(1) );
      break;
    }
    default: e->ignore();
  }
  QMainWindow::keyPressEvent( e );
}

void wordList::slDoubleClicked( QListViewItem *item, const QPoint & , int )
{
   if ( item ) emit selected( item->text(0),item->text(1) );
}

////////////////////////  setup dialog  ///////////////

setup::setup( QWidget *parent, const char *name )
  :  QDialog( parent, name )
{
  codec = QTextCodec::codecForName("KOI8-R");
  setCaption( i18n("Preferences") );
  
  QGridLayout *grid = new QGridLayout( this, 6, 2, 2, 2 );
  
  QLabel *label = new QLabel( this );
  label->setText( "Applet Setup" );
  label->setAlignment( Qt::AlignCenter );
  label->setFrameStyle( QFrame::Plain | QFrame::Panel );
  label->setPaletteBackgroundColor( QColor( 193, 255, 200 ) );
  grid->addMultiCellWidget( label, 0, 0, 0, 1 );
  
  lvDicts = new QListView( this );
  lvDicts->addColumn( i18n("Dictionaries") );
  lvDicts->setRootIsDecorated( TRUE );
  grid->addWidget( lvDicts, 1, 0 );
  
  QPushButton *pb = new QPushButton( i18n("Dictionaries Directory"), this );
  connect( pb, SIGNAL( clicked() ), this, SLOT( slotSetDict() ) );
  grid->addWidget( pb, 1, 1, Qt::AlignTop );
  
  leWords = new QLineEdit( this );
  grid->addWidget( leWords, 2, 0 );
  pb = new QPushButton( i18n("Sounds Directory"), this );
  connect( pb, SIGNAL( clicked() ), this, SLOT( slotSetDir() ) );
  grid->addWidget( pb, 2, 1 );
  
  QLabel *lb = new QLabel( this );
  lb->setText( i18n( "Make kind of index file for Muller Dictionary" ) );
  grid->addWidget( lb, 3, 0 );
  
  pb = new QPushButton( i18n("Make Index"), this );
  connect( pb, SIGNAL( clicked() ), this, SLOT( slotIndex() ) );
  grid->addWidget( pb, 3, 1 );
  QToolTip::add( pb, i18n("This options is for priveleged users only") );
    
  cbRegExp = new QCheckBox( i18n("Use Regular Expression"), this );
  grid->addMultiCellWidget( cbRegExp, 4,4,0,1 );
  
  QHBox *hb = new QHBox( this );
  pb = new QPushButton( i18n("Ok"), hb );
  connect( pb, SIGNAL( clicked() ), this, SLOT( slotOkPressed() ) );
  pb->setDefault( TRUE );
  
  pb = new QPushButton( i18n("Cancel"), hb );
  connect( pb, SIGNAL( clicked() ), this, SLOT( slotCancelPressed() ) );
  grid->addMultiCellWidget( hb, 5,5,0,1 );
  
  QSettings s;

  QListViewItem *item = new QListViewItem( lvDicts );
  item->setText( 0,s.readEntry( "/reer/main/dictdir", "/" ) );
  
  setDictsList( item );
  
  leWords->setText( s.readEntry( "/reer/main/dir", "/" ) );
  cbRegExp->setChecked( s.readBoolEntry( "/reer/main/useRegExp", FALSE ) );

  resize( QSize( s.readNumEntry( "/reer/setup/width", 500 ),
                 s.readNumEntry( "/reer/setup/height", 400 ) ) );    
}

setup::~setup()
{
  QSettings s;
  s.writeEntry( "/reer/setup/width", width() );
  s.writeEntry( "/reer/setup/height", height() );
}

void setup::slotSetDict()
{
  KURL url = KDirSelectDialog::selectDirectory( lvDicts->firstChild()->text(0), TRUE );
  
  lvDicts->clear();
  QListViewItem *item = new QListViewItem( lvDicts );
  
  if ( !url.isEmpty() ) {
    item->setText( 0, url.path() );
  }else{
    item->setText( 0, "/usr/local/share/dict" );
  }
  setDictsList( item );
}

void setup::setDictsList( QListViewItem *item )
{
  QSettings s;
  QStringList lst = s.readListEntry( "/reer/main/active_dicts" );
  
  QDir dir( item->text(0), "*.dict" );
  if ( dir.exists() ) {
    for ( uint i=0; i<dir.count(); i++ ) {
      QCheckListItem *subItem = new QCheckListItem( item, dir[i], QCheckListItem::CheckBox );
      if ( ! lst.grep( QRegExp(dir[i]) ).empty() ) subItem->setOn( TRUE );
    }
  }
  item->setOpen( TRUE );
}

void setup::slotSetDir()
{
  KURL url = KDirSelectDialog::selectDirectory( leWords->text(), TRUE );
  if ( !url.isEmpty() ) {
    leWords->setText( url.path() );
  }else{
    leWords->setText( "/usr/local/share/erss/sounds" );
  }
}


void setup::slotOkPressed()
{
  QSettings s;
  s.writeEntry( "/reer/main/dictdir", lvDicts->firstChild()->text(0) );
  QStringList lst;
  QCheckListItem *item = static_cast<QCheckListItem*> ( lvDicts->firstChild()->firstChild() );
  while( item ) {
    if( item->isOn() ) lst << item->text();
    item = static_cast<QCheckListItem*> ( item->nextSibling() );
  }
  s.writeEntry( "/reer/main/active_dicts", lst );
  s.writeEntry( "/reer/main/dir", leWords->text() );
  s.writeEntry( "/reer/main/useRegExp", cbRegExp->isChecked() );
  accept();
}

void setup::slotCancelPressed()
{
  reject();
}

void setup::slotIndex()
{
  QStringList lst;
  QFile file;
  QFile index;
  QString fn = KFileDialog::getOpenFileName("/","*.dict");
  if ( fn.isEmpty() ) return;
  file.setName( fn );
//  index.setName( leDict->text()+".index" );
  index.setName( fn+".index" );
  QTextStream stream( &index );
  if ( file.open( IO_ReadOnly ) && index.open( IO_WriteOnly ) ) {
    QProgressDialog progress( "Indexing dictionary...", 0, file.size(), this, "progress", TRUE );
    while ( !file.atEnd() ) {
      QString str;
      if ( file.readLine( str, file.size() )>0 ) {
        stream << codec->toUnicode( str.left( str.find(QRegExp("  "),0) ) ) <<"@"<< file.at()-str.length() << '\n';
      }
      progress.setProgress( file.at() );
     }
    progress.setProgress( file.size() );
    file.close();
    index.close();
  }
}

////////////////////////   myListViewItem   ///////////////////////

myListViewItem::myListViewItem( QListView *parent )
  :QListViewItem( parent )
{
}

myListViewItem::myListViewItem( QListView *parent, QListViewItem *after )
  :QListViewItem( parent, after )
{
}

myListViewItem::~myListViewItem()
{
}

void myListViewItem::paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int alignment )
{
    QColorGroup _cg( cg );
    QColor c = _cg.text();

    if ( column==1 ) {
      int h,s,v;
      c.getHsv( &h, &s, &v );
      v = v+(255-v)/2;
      QColor cl;
      cl.setHsv( h, s, v );
      _cg.setColor( QColorGroup::Text, cl );
   }
    QListViewItem::paintCell( p, _cg, column, width, alignment );

    _cg.setColor( QColorGroup::Text, c );
}

///////////////////////////    maDictionary     ////////////////////////

maDictionary::maDictionary()
{
}

maDictionary::maDictionary( QString n )
{
  name = n;
  loadIndex();
}

maDictionary::~maDictionary()
{
}

void maDictionary::loadIndex()
{
  index.clear();
  QFile fi;
  fi.setName( name+".index" );
  QTextStream ts( &fi );
  ts.setEncoding( QTextStream::UnicodeUTF8 );
  if ( fi.open( IO_ReadOnly ) ) {
    while ( !fi.atEnd() ) {
      QString str=ts.readLine();
      if ( str!="" ) index << str.stripWhiteSpace();
    }
    fi.close();
  }
}

void maDictionary::setName( QString n )
{
  name = n;
  loadIndex();
}

QStringList maDictionary::grep( QRegExp rx )
{
  QStringList lst = index.grep( rx );
  lst.gres( QRegExp( "^(.*)@(.*)$" ), "\\1#"+QFileInfo( name ).fileName() );
  return lst;
}

QStringList maDictionary::grepp( QRegExp rx )
{
  return index.grep( rx );
}
